/******************************************************************************
 * BackgammonBoard.java - Be a backgammon board
 * $Id: BackgammonBoard.java,v 1.8 2011/06/07 19:42:57 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: BackgammonBoard.java,v $
 * Revision 1.8  2011/06/07 19:42:57  dick
 * onHome/Bar 1/2 becomes onHomeBar X/O
 * outBoard() takes a Player, not a JibsWriter.  -- We need to know what style the Player wants.
 * Finish crawford attributes.
 * switch2O needs to set the reverse players.
 * isRace() is a helper for greedy mode.
 *
 * Revision 1.7  2011/06/05 05:56:01  dick
 * Get rid of Player1 and 2, and MayDouble1 and 2.  Call them PlayerX and PlayerO and MayDoubleX and O.
 *
 * Revision 1.6  2011/06/03 19:26:10  dick
 * Add some debug opening boards.
 *
 * Revision 1.5  2011/06/03 17:05:48  dick
 * Define X=0 and O=1 so we can get playerScore[X] and playerScore[O].
 *
 * Revision 1.4  2011/06/03 04:14:07  dick
 * Attributes player[12]Got becomes player[12]Score.
 *
 * Revision 1.3  2011/06/01 20:26:18  dick
 * Deadwood delete.  Privatize privates.  Javadoc.
 *
 * Revision 1.2  2011/05/30 05:29:21  dick
 * retab.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.backgammon;

import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.server.Player;
import net.sourceforge.jibs.util.JibsSet;

/**
 * This is the encapsulation of a backgammon board as needed for <b>Jibs</b>.<br>
 * Therefore it also models a match between two players of several games.
 * Each game uses a <i>real</i> Backgammon board to represent such a
 * particular game.<br>
 * One of the players is playing the X-pieces (represemted by negative
 * numbers) whereas the other is playing the O-pieces (represented by
 * positive numbers).<br>
 * <br>
 * This class uses an array of 26 ints to represemt the board internally.
 * 
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.8 $ <br> $Date: 2011/06/07 19:42:57 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/backgammon/BackgammonBoard.java">cvs BackgammonBoard.java</a>
 */
public class BackgammonBoard {
	/** Index into the X player */
	public static final int X = 0;
	/** Index into the O player */
	public static final int O = 1;

	//	private static Logger logger = Logger.getLogger(BackgammonBoard.class);
	char[] value = new char[] {13,10};
	public String newline = new String(value);
	private JibsServer jibsServer;
	private Player playerX;
	private Player playerO;
	private Die pDie1;
	private Die pDie2;
	private Die oDie1;
	private Die oDie2;
	private int bar;
	private int[] board;
	private int canMove;
	private int color;
	private int cubeNumber;
	private int didCrawford;
	private int direction;
	private int forcedMove;
	private int home;
	private int matchlength;
	private int mayDoubleX;
	private int mayDoubleO;
	private int onBarX;
	private int onBarO;
	private int onHomeX;
	private int onHomeO;
	int[] playerScore = new int[2];
	String[] playerName = new String[2];
	private int redoubles;
	private int turn;
	private int wasDoubled;
	private JibsMatch jibsMatchVersion;
	private Player joinPlayer1;
	private Player joinPlayer2;
	private boolean crawFordGame;
	private	boolean	postCrawfordGame;
	private boolean isEnded;

	/** Construct a clone of the passed board
	 * @param b2 The board to make a copy of.
	 */
	public BackgammonBoard(final BackgammonBoard b2) {
		this.init(b2.jibsServer, b2.playerX, b2.playerO,	// why init when everything is overridden?
				b2.matchlength, b2.getMatchVersion());
		this.jibsServer = b2.jibsServer;
		this.playerX = b2.playerX;
		this.playerO = b2.playerO;
		this.pDie1.setJibsServer(b2.pDie1.getJibsServer());
		this.pDie2.setJibsServer(b2.pDie2.getJibsServer());
		this.oDie1.setJibsServer(b2.oDie1.getJibsServer());
		this.oDie2.setJibsServer(b2.oDie2.getJibsServer());
		this.pDie1.setValue(b2.pDie1.getValue());
		this.pDie2.setValue(b2.pDie2.getValue());
		this.oDie1.setValue(b2.oDie1.getValue());
		this.oDie2.setValue(b2.oDie2.getValue());
		this.bar = b2.bar;
		this.canMove = b2.canMove;
		this.color = b2.color;
		this.cubeNumber = b2.cubeNumber;
		this.didCrawford = b2.didCrawford;
		this.direction = b2.direction;
		this.forcedMove = b2.forcedMove;
		this.home = b2.home;
		this.matchlength = b2.matchlength;
		this.mayDoubleX = b2.mayDoubleX;
		this.mayDoubleO = b2.mayDoubleO;
		this.onBarX = b2.onBarX;
		this.onBarO = b2.onBarO;
		this.onHomeX = b2.onHomeX;
		this.onHomeO = b2.onHomeO;
//		this.playerName[X] = b2.playerName[X];
//		this.playerName[O] = b2.playerName[O];
		this.playerScore[O] = b2.playerScore[O];
		this.playerScore[X] = b2.playerScore[X];
		this.redoubles = b2.redoubles;
		this.turn = b2.turn;
		this.wasDoubled = b2.wasDoubled;
		this.jibsMatchVersion = b2.jibsMatchVersion;
		this.joinPlayer1 = b2.joinPlayer1;
		this.joinPlayer2 = b2.joinPlayer2;
		this.crawFordGame = b2.crawFordGame;
		this.postCrawfordGame = b2.postCrawfordGame;

		for (int i = 0; i < 26; i++) {
			this.board[i] = b2.board[i];
		}

		this.isEnded = b2.isEnded;
	}

	/** Construct a board with these values.
	 * @param jibsServer
	 * @param playerXName
	 * @param playerOName
	 * @param matchlength
	 * @param matchVersion
	 */
	public BackgammonBoard(JibsServer jibsServer, Player playerX, Player playerO,
			int matchlength, JibsMatch matchVersion) {
		this.init(jibsServer, playerX, playerO, matchlength,
				matchVersion);
	}

	/**
	 */
	private void init(JibsServer jibsServer, Player playerX, Player playerO,
			int matchlength, JibsMatch matchVersion) {
		this.playerX = playerX;
		this.playerO = playerO;
		this.playerName[X] = playerX.getName();
		this.playerName[O] = playerO.getName();
		this.matchlength = matchlength;
		this.jibsMatchVersion = matchVersion;

		pDie1 = new Die(jibsServer);
		pDie2 = new Die(jibsServer);
		oDie1 = new Die(jibsServer);
		oDie2 = new Die(jibsServer);

		cubeNumber = 1;
		playerScore[O] = 0;
		playerScore[X] = 0;
		board = new int[26];

		// standard backgammon board
		for (int i = 0; i < board.length; i++) {
			board[i] = 0;
		}

		// Standard opening
		board[24] = 2;
		board[13] = 5;
		board[8] = 3;
		board[6] = 5;
		board[1] = -2;
		board[12] = -5;
		board[17] = -3;
		board[19] = -5;

/*
		board[1]=-2; board[3] = 2; board[4] = 2; board[6] = 3; board[8] = 1;
		board[12] = -4; board[13] = 5; board[17] = -4; board[19] = -5;
		board[24] = 2; //board[0]=2; // Bar von 'X' //board[25]=2; // Bar von 'O'
*/
/*		// Debug for a gammon
		board[1] = 4;
		board[4] = -5;
		board[5] = -5;
		board[6] = -5;
*/
/*		// Debug for a gammon the other way
		board[24] = -4;
		board[23] = 5;
		board[22] = 5;
		board[21] = 5;
*/
		forcedMove = 0;
		didCrawford = 0;
		redoubles = 0;
		mayDoubleX = 1;
		mayDoubleO = 1;
		wasDoubled = 0;
		color = -1;
		direction = 1;
		home = 25;
		bar = 0;

		// board[0]=2; // Bar 'X'
		int totalPlayer1 = board[0];
		int totalPlayer2 = board[25];

		for (int i = 1; i <= 24; i++) {
			if (board[i] > 0) {
				totalPlayer2 += Math.abs(board[i]);
			}

			if (board[i] < 0) {
				totalPlayer1 += Math.abs(board[i]);
			}
		}

		onHomeX = 15 - totalPlayer1;
		onHomeO = 15 - totalPlayer2;

		onBarX = board[0];
		onBarO = board[25];
		this.isEnded = false;
	}

	/**
	 *
	 * @return
	 */
	public int getBar() {
		return bar;
	}

	/**
	 *
	 * @return
	 */
	public int[] getBoard() {
		return board;
	}

	/**
	 *
	 * @return
	 */
	public int getCanMove() {
		return canMove;
	}

	/**
	 *
	 * @return
	 */
	public int getColor() {
		return color;
	}

	/**
	 *
	 * @return
	 */
	public int getCubeNumber() {
		return cubeNumber;
	}

	/**
	 *
	 * @return
	 */
	public int getDidCrawford() {
		return didCrawford;
	}

	/**
	 *
	 * @return
	 */
	public int getDirection() {
		return direction;
	}

	/**
	 *
	 */
	public int getForcedMove() {
		return forcedMove;
	}

	/**
	 *
	 * @return
	 */
	public int getHome() {
		return home;
	}

	/**
	 *
	 * @return
	 */
	public int getMatchlength() {
		return matchlength;
	}

	/** Is PlayerX allowed to double
	 * @return 1 = yes, 0 = no
	 */
	public int getMayDoubleX() {
		return mayDoubleX;
	}

	/** Is PlayerX allowed to double
	 * @return 1 = yes, 0 = no
	 */
	public int getMayDoubleO() {
		return mayDoubleO;
	}

	public boolean getMayDouble(Player player) {
		if (player == playerX)
			return(mayDoubleX == 1);
		if (player == playerO)
			return(mayDoubleO == 1);
		throw new RuntimeException("Can't match player: " + player.getName());
		
	}
	/**
	 *
	 * @return
	 */
	public int getOnBarX() {
		return onBarX;
	}

	/**
	 *
	 * @return
	 */
	public int getOnBarO() {
		return onBarO;
	}

	/**
	 *
	 * @return
	 */
	public int getOnHomeX() {
		return onHomeX;
	}

	/**
	 *
	 * @return
	 */
	public int getOnHomeO() {
		return onHomeO;
	}

	public int getOnHome(int XO) {
		if (XO == X)
			return(onHomeX);
		return(onHomeO);
	}
	/** Get a boardstyle=3 board of the current state of the game
	 * @param game The game to turn into a string
	 * @return A Fibs type 3 board.
	 */
	public String getOutputBoard(JibsGame game) {
		StringBuffer strBoard = new StringBuffer();

		strBoard.append("board:");
		strBoard.append("You:");
		strBoard.append(playerName[O]);
		strBoard.append(":");
		strBoard.append(matchlength);
		strBoard.append(":");
		strBoard.append(playerScore[X]);
		strBoard.append(":");
		strBoard.append(playerScore[O]);

		for (int i = 0; i < 26; i++) {
			strBoard.append(":");
			strBoard.append(board[i]);
		}

		strBoard.append(":");
		strBoard.append(turn);
		strBoard.append(":");
		strBoard.append(getpDie1().getValue());
		strBoard.append(":");
		strBoard.append(getpDie2().getValue());
		strBoard.append(":");
		strBoard.append(getoDie1().getValue());
		strBoard.append(":");
		strBoard.append(getoDie2().getValue());
		strBoard.append(":");
		strBoard.append(cubeNumber);
		strBoard.append(":");
		strBoard.append(mayDoubleX);
		strBoard.append(":");
		strBoard.append(mayDoubleO);
		strBoard.append(":");
		strBoard.append(wasDoubled);
		strBoard.append(":");
		strBoard.append(color);
		strBoard.append(":");
		strBoard.append(direction);
		strBoard.append(":");
		strBoard.append(home);
		strBoard.append(":");
		strBoard.append(bar);
		strBoard.append(":");
		strBoard.append(onHomeX);
		strBoard.append(":");
		strBoard.append(onHomeO);
		strBoard.append(":");
		strBoard.append(onBarX);
		strBoard.append(":");
		strBoard.append(onBarO);
		strBoard.append(":");
		strBoard.append(canMove);
		strBoard.append(":");
		strBoard.append(forcedMove);
		strBoard.append(":");
		strBoard.append(didCrawford);
		strBoard.append(":");
		strBoard.append(redoubles);

		return strBoard.toString();
	}

	/**
	 *
	 * @return
	 */
	public int getRedoubles() {
		return redoubles;
	}

	/** Who's turn is it?
	 *
	 * @return -1 = X, 1 = O
	 */
	public int getTurn() {
		return turn;
	}

	/**
	 *
	 * @return
	 */
	public int getWasDoubled() {
		return wasDoubled;
	}

	/**
	 *
	 * @param player The player to write to
	 * @param name The name to write to the player
	 * @param turnXO Is it PlayerX or PlayerO's turn
	 * @param die11 PlayerX first die
	 * @param die12 PlayerX second die
	 * @param die21 PlayerO first die
	 * @param die22 PlayerO second die
	 */
	public void outBoard(Player player, String name, int turnXO,
			int die11, int die12, int die21, int die22) {
		JibsSet jibsSet = player.getJibsSet();
		String strBoardStyle = (String) jibsSet.get("boardstyle");
		int boardstyle = new Integer(strBoardStyle).intValue();

		switch (boardstyle) {
		case 1:default:
		case 2:
			outBoard_2(player, turnXO, die11, die12, die21, die22);

			break;

		case 3:

			String x = outBoard_3(player, name, turnXO, die11, die12, die21, die22);
			player.println(x);

			break;
		}
	}

	/**
	 *
	 * @param out1
	 * @param turn
	 * @param dice11
	 * @param dice12
	 * @param dice21
	 * @param dice22
	 */
	private void outBoard_2(Player player, int turn, int dice11, int dice12,
			int dice21, int dice22) {
		int lines;
		int i;
		StringBuffer bf = new StringBuffer();

		bf.append(newline);
		bf.append("+-1--2--3--4--5--6-------7--8--9-10-11-12-+");
		bf.append(" o:");
		bf.append(playerName[O]);
		bf.append(" - score: ");
		bf.append(playerScore[O]);

		for (lines = 1; lines <= 5; lines++) {
			bf.append(newline);
			bf.append("|");

			for (i = 1; i <= 6; i++) {
				int which = board[i];
				String symbol = "   ";

				if (which < 0) {
					symbol = " x ";
				} else {
					if (which > 0) {
						symbol = " o ";
					}
				}

				if (Math.abs(which) >= lines) {
					bf.append(symbol);
				} else {
					symbol = "   ";
					bf.append(symbol);
				}
			}

			bf.append("|   |");

			for (i = 7; i <= 12; i++) {
				int which = board[i];
				String symbol = "   ";

				if (which < 0) {
					symbol = " x ";
				} else {
					if (which > 0) {
						symbol = " o ";
					}
				}

				if (Math.abs(which) >= lines) {
					bf.append(symbol);
				} else {
					symbol = "   ";
					bf.append(symbol);
				}
			}

			bf.append("|");
		}

		bf.append(newline);
		bf.append("|                  |BAR|                  | v ");

		if (getMatchVersion().getVersion() == JibsMatch.nPointMatch) {
			bf.append(matchlength);
			bf.append("-point match");
		} else {
			bf.append("unlimited match");
		}

		for (lines = 5; lines >= 1; lines--) {
			bf.append("\r\n|");

			for (i = 24; i >= 19; i--) {
				int which = board[i];
				String symbol = "   ";

				if (which < 0) {
					symbol = " x ";
				} else {
					if (which > 0) {
						symbol = " o ";
					}
				}

				if (Math.abs(which) >= lines) {
					bf.append(symbol);
				} else {
					symbol = "   ";
					bf.append(symbol);
				}
			}

			bf.append("|   |");

			for (i = 18; i >= 13; i--) {
				int which = board[i];
				String symbol = "   ";

				if (which < 0) {
					symbol = " x ";
				} else {
					if (which > 0) {
						symbol = " o ";
					}
				}

				if (Math.abs(which) >= lines) {
					bf.append(symbol);
				} else {
					symbol = "   ";
					bf.append(symbol);
				}
			}

			bf.append("|");
		}

		bf.append(newline);
		bf.append("+24-23-22-21-20-19------18-17-16-15-14-13-+");
		bf.append(" x:");
		bf.append(playerName[X]);
		bf.append(" - score: ");
		bf.append(playerScore[X]);
		bf.append(newline);
		bf.append("BAR: O:");
		bf.append(onBarO);
		bf.append(" X:");
		bf.append(onBarX);
		bf.append("   OFF: O:");
		bf.append(onHomeO);
		bf.append(" X:");
		bf.append(onHomeX);
		bf.append("   Cube: ");
		bf.append(cubeNumber);

		if (dice11 > 0) {
			bf.append("   You rolled " + dice11 + " and " + dice12 + ".");
		}

		player.println(bf.toString());
	}

	/**
	 *
	 * @param out1
	 * @param turn
	 * @param dice11
	 * @param dice12
	 * @param dice21
	 * @param dice22
	 * @return
	 */
	private String outBoard_3(Player player, String name, int turn,
			int dice11, int dice12, int dice21, int dice22) {
		StringBuffer strBoard = new StringBuffer();

		strBoard.append("board:");
		strBoard.append(name);
		strBoard.append(":");
		strBoard.append(getPlayerOName());
		strBoard.append(":");

		if (getMatchVersion().getVersion() == JibsMatch.nPointMatch) {
			strBoard.append(matchlength);
		} else {
			strBoard.append("9999");
		}

		strBoard.append(":");
		strBoard.append(playerScore[X]);
		strBoard.append(":");
		strBoard.append(playerScore[O]);

		for (int i = 0; i < 26; i++) {
			strBoard.append(":");
			strBoard.append(board[i]);
		}

		strBoard.append(":");
		strBoard.append(turn);
		strBoard.append(":");
		strBoard.append(dice11);
		strBoard.append(":");
		strBoard.append(dice12);
		strBoard.append(":");
		strBoard.append(dice21);
		strBoard.append(":");
		strBoard.append(dice22);
		strBoard.append(":");
		strBoard.append(cubeNumber);
		strBoard.append(":");
		strBoard.append(mayDoubleX);
		strBoard.append(":");
		strBoard.append(mayDoubleO);
		strBoard.append(":");
		strBoard.append(wasDoubled);
		strBoard.append(":");
		strBoard.append(color);
		strBoard.append(":");
		strBoard.append(direction);
		strBoard.append(":");
		strBoard.append(home);
		strBoard.append(":");
		strBoard.append(bar);
		strBoard.append(":");
		strBoard.append(onHomeX);
		strBoard.append(":");
		strBoard.append(onHomeO);
		strBoard.append(":");
		strBoard.append(onBarX);
		strBoard.append(":");
		strBoard.append(onBarO);
		strBoard.append(":");
		strBoard.append(canMove);
		strBoard.append(":");
		strBoard.append(forcedMove);
		strBoard.append(":");
		strBoard.append(didCrawford);
		strBoard.append(":");
		strBoard.append(redoubles);

		return strBoard.toString();
	}

	/**
	 *
	 * @param mv
	 * @return
	 */
	public BackgammonBoard placeMoveO(Move mv) {
		BackgammonBoard helpBoard = new BackgammonBoard(this);
		int source = mv.source;
		int destination = mv.destination;

		if (source == 25) { // moving from bar
			helpBoard.board[0]--;
			helpBoard.onBarO--;
		} else {
			helpBoard.board[source]--;
		}

		if (destination <= 0) {
			// bearing off
			helpBoard.onHomeO++;
		} else {
			helpBoard.board[destination]++;

			if (helpBoard.board[destination] == 0) {
				helpBoard.board[destination]++;
				helpBoard.board[0]++;
				helpBoard.onBarX++;
			}

			if (destination <= 0) {
				helpBoard.onHomeO++;
			}
		}

		return helpBoard;
	}

	/**
	 * Perform a move on this board.
	 *
	 * @param mv
	 *            a half-move
	 *
	 * @return A modified backgammon board where the move has been placed on.
	 */
	public BackgammonBoard placeMoveX(Move mv) {
		BackgammonBoard helpBoard = new BackgammonBoard(this);
		int source = mv.source;
		int destination = mv.destination;

		if (source == bar) { // moving from bar	// XXX: This seems wrong.  Using bar as a const?? maybe?
			helpBoard.board[0]--;
			helpBoard.onBarX--;
		} else if (source >= 1 && source <= 24) {
			helpBoard.board[source]++;
		}

		if (destination <= 0) {
			// bearing off
			helpBoard.onHomeX++;
		} else {
			if ((destination >= 1) && (destination <= 24)) {
				helpBoard.board[destination]--;

				if (helpBoard.board[destination] == 0) {
					helpBoard.board[destination]--;
					helpBoard.board[25]++;
					helpBoard.onBarO++;
				}
			}

			if (destination >= 25) {
				helpBoard.onHomeX++;
			}
		}

		return helpBoard;
	}

	/**
	 *
	 * @param bar
	 */
	public void setBar(int bar) {
		this.bar = bar;
	}

	/**
	 *
	 * @param board
	 */
	public void setBoard(int[] board) {
		this.board = board;
	}

	/**
	 *
	 * @param canMove
	 */
	public void setCanMove(int canMove) {
		this.canMove = canMove;
	}

	/**
	 *
	 * @param color
	 */
	public void setColor(int color) {
		this.color = color;
	}

	/**
	 *
	 * @param cubeNumber
	 */
	public void setCubeNumber(int cubeNumber) {
		this.cubeNumber = cubeNumber;
	}

	/**
	 *
	 * @param didCrawford
	 */
	public void setDidCrawford(int didCrawford) {
		this.didCrawford = didCrawford;
	}

	/**
	 *
	 * @param direction
	 */
	public void setDirection(int direction) {
		this.direction = direction;
	}

	/**
	 *
	 * @param forcedMove
	 */
	public void setForcedMove(int forcedMove) {
		this.forcedMove = forcedMove;
	}

	/**
	 *
	 * @param home
	 */
	public void setHome(int home) {
		this.home = home;
	}

	/**
	 *
	 * @param matchlength
	 */
	public void setMatchlength(int matchlength) {
		this.matchlength = matchlength;
	}

	/** Can PlayerX double?
	 * @param mayDouble 1 = true
	 */
	public void setMayDoubleX(int mayDouble) {
		this.mayDoubleX = mayDouble;
	}

	/** Can PlayerO double?
	 * @param mayDouble 1 = true
	 */
	public void setMayDoubleO(int mayDouble) {
		this.mayDoubleO = mayDouble;
	}

	/**
	 *
	 * @param onBar1
	 */
	public void setOnBarX(int onBarX) {
		this.onBarX = onBarX;
	}

	/**
	 *
	 * @param onBar2
	 */
	public void setOnBarO(int onBarO) {
		this.onBarO = onBarO;
	}

	/**
	 *
	 * @param onHome1
	 */
	public void setOnHomeX(int onHomeX) {
		this.onHomeX = onHomeX;
	}

	/**
	 *
	 * @param onHome2
	 */
	public void setOnHomeO(int onHome2) {
		this.onHomeO = onHome2;
	}

	/**
	 *
	 * @param player1Score
	 */
	public void setPlayerScore(int XO, int playerScore) {
		this.playerScore[XO] = playerScore;
	}

	/** Add these points to player1's score.
	 * @param XO These points go to either player X or player O
	 * @param points The number of points he won this game.
	 */
	public void addPlayerScore(int XO, int points) {
		this.playerScore[XO] += points;
	}

	/** Get one of the players scores
	 * @param XO Do we want player X or player O's score
	 * @return Player X's or Player O's score
	 */
	public int getPlayerScore(int XO) {
		return(this.playerScore[XO]);
	}

	/**
	 *
	 * @param score
	 */
	public void setPlayerXScore(int score) {
		playerScore[X] = score;
	}

	/**
	 *
	 * @param score
	 */
	public void setPlayerOScore(int score) {
		playerScore[O] = score;
	}

	/**
	 *
	 * @return
	 */
	public String getPlayerXName() {
		return playerName[X];
	}

	/**
	 *
	 * @param player1Name
	 */
	public void setPlayerXName(String player1Name) {
		this.playerName[X] = player1Name;
	}


	/**
	 *
	 * @return
	 */
	public String getPlayerOName() {
		return playerName[O];
	}


	/**
	 *
	 * @param player2Score
	 */
//	public void setPlayer2Score(int player2Score) {
//		this.player2Score = player2Score;
//	}

	/** Add these points to player2's score.
	 * @param points The number of points he won this game.
	 */
//	public void addPlayer2Score(int points) {
//		this.player2Score += points;
//	}

	/**
	 *
	 * @param playerOName
	 */
	public void setPlayerOName(String playerOName) {
		this.playerName[O] = playerOName;
	}

	/**
	 *
	 * @param redoubles
	 */
	public void setRedoubles(int redoubles) {
		this.redoubles = redoubles;
	}

	/**
	 *
	 * @param turn
	 */
	public void setTurn(int turn) {
		this.turn = turn;
	}

	/**
	 *
	 * @param wasDoubled
	 */
	public void setWasDoubled(int wasDoubled) {
		this.wasDoubled = wasDoubled;
	}

	/**
	 *
	 * @param matchVersion
	 */
	public void setMatchVersion(JibsMatch matchVersion) {
		this.jibsMatchVersion = matchVersion;
	}

	/**
	 *
	 * @param d1
	 */
	public void setpDie1(Die d1) {
		pDie1.setValue(d1.getValue());
	}

	/**
	 *
	 * @return
	 */
	public Die getpDie1() {
		return pDie1;
	}

	/**
	 *
	 * @param d2
	 */
	public void setpDie2(Die d2) {
		pDie2.setValue(d2.getValue());
	}

	/**
	 *
	 * @return
	 */
	public Die getpDie2() {
		return pDie2;
	}

	/**
	 *
	 * @return
	 */
	public Player getPlayerX() {
		return playerX;
	}

	/**
	 *
	 * @param playerX
	 */
	public void setPlayerX(Player playerX) {
		this.playerX = playerX;
	}

	/**
	 *
	 * @param playerO
	 */
	public void setPlayerO(Player playerO) {
		this.playerO = playerO;
	}

	/**
	 *
	 * @return
	 */
	public Player getPlayerO() {
		return playerO;
	}

	/**
	 *
	 * @param die
	 */
	public void setoDie1(Die die) {
		oDie1.setValue(die.getValue());
	}

	/**
	 *
	 * @return
	 */
	public Die getoDie1() {
		return oDie1;
	}

	/**
	 *
	 * @return
	 */
	public Die getoDie2() {
		return oDie2;
	}

	/**
	 *
	 * @param die
	 */
	public void setoDie2(Die die) {
		oDie2.setValue(die.getValue());
	}

	/**
	 *
	 * @return
	 */
	public JibsMatch getMatchVersion() {
		return jibsMatchVersion;
	}

	/**
	 *
	 * @param player
	 * @return
	 */
	public Player getOpponent(Player player) {
		if (player.getName().equals(playerX.getName())) {
			return playerO;
		}

		if (player.getName().equals(playerO.getName())) {
			return playerX;
		}

		return null;
	}

	/**
	 *
	 * @return
	 */
	public int getPlayerXScore() {
		return playerScore[X];
	}

	/**
	 *
	 * @return
	 */
	public int getPlayerOScore() {
		return playerScore[O];
	}

	/** Is this Player playing as PlayerX
	 * @param player The Player to test
	 * @return true if he is PlayerX, false if he is PlayerO
	 */
	public boolean isPlayerX(Player player) {
		return(player == this.playerX);
		/*
		if (player.getName().equals(this.playerX.getName()))
			return true;
		else
			return false;
		*/
	}

	/**
	 *
	 * @return
	 */
	public boolean isEnded() {
		return isEnded;
	}

	public void setEnded(boolean isEnded) {
		this.isEnded = isEnded;
	}

	/**
	 *
	 * @param player
	 */
	public void setJoinPlayer(Player player) {
		if (joinPlayer1 == null) {
			joinPlayer1 = player;
		} else {
			joinPlayer2 = player;
		}
	}

	/**
	 *
	 * @return
	 */
	public Object getJoinPlayer1() {
		return joinPlayer1;
	}

	/**
	 *
	 * @return
	 */
	public Object getJoinPlayer2() {
		return joinPlayer2;
	}

	public boolean isPostCrawfordGame() {
		return(this.postCrawfordGame);
	}
	/**
	 *
	 * @param b
	 */
	public void setPostCrawFordGame(boolean b) {
		this.postCrawfordGame = b;
	}

	/**
	 *
	 * @return
	 */
	public boolean isCrawFordGame() {
		return(this.crawFordGame);
	}

	/**
	 *
	 * @param b
	 */
	public void setCrawFordGame(boolean b) {
		crawFordGame = b;
	}

	/**
	 *
	 * @param object
	 */
	public void setJoinPlayer1(Player object) {
		joinPlayer1 = object;
	}

	/**
	 *
	 * @param object
	 */
	public void setJoinPlayer2(Player object) {
		joinPlayer2 = object;
	}

	/** Create a copy of this board from the opponent's point of view.
	 * @return The mirror of this board
	 */
	public BackgammonBoard switch2O() {
		BackgammonBoard opBoard = new BackgammonBoard(this);
		opBoard.setPlayerXName(getPlayerOName());
		opBoard.setPlayerOName(getPlayerXName());
		opBoard.setPlayerX(getPlayerO());
		opBoard.setPlayerO(getPlayerX());
		opBoard.setDirection(getDirection() == 1 ? -1 : 1);
		opBoard.setColor(getColor() == 1 ? 0 : 1);
		opBoard.getpDie1().setValue(getoDie1().getValue());
		opBoard.getpDie2().setValue(getoDie2().getValue());
		opBoard.getoDie1().setValue(getpDie1().getValue());
		opBoard.getoDie2().setValue(getpDie2().getValue());
		opBoard.setOnBarX(getOnBarO());
		opBoard.setOnBarO(getOnBarX());
		opBoard.setOnHomeX(getOnHomeO());
		opBoard.setOnHomeO(getOnHomeX());
		opBoard.setBar(getHome());
		opBoard.setHome(getBar());
		opBoard.setPlayerXScore(getPlayerOScore());
		opBoard.setPlayerOScore(getPlayerXScore());
		opBoard.setMayDoubleX(getMayDoubleO());
		opBoard.setMayDoubleO(getMayDoubleX());

		return opBoard;
	}

	/** Is this game a race?
	 * Determine whether all of one Player's checkers are past the opponent's checkers
	 * @return true if this game has turned into a race.
	 */
	public boolean isRace() {
		int i;
		int k = 0;
		if (onBarO > 0 || onBarX > 0)
			return(false);
		boolean oppositeSeen = false;
		for (i=1; i<=24; i++) {
			if (this.board[i] == 0)
				continue;
			if (k == 0) {
				k = this.board[i];
				continue;
			}
			if (!oppositeSeen) {
				if (k < 0 && this.board[i] > 0) {
					oppositeSeen = true;
					continue;
				}
				if (k > 0 && this.board[i] < 0) {
					oppositeSeen = true;
					continue;
				}
			} else {
				if (k < 0 && this.board[i] < 0)
					return(false);
				if (k > 0 && this.board[i] > 0)
					return(false);
			}
		}
		return(true);
	}

}
