/******************************************************************************
 * Reject_Command - Player rejects the other guy's resignation
 * $Id: Reject_Command.java,v 1.6 2011/06/06 03:18:25 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Reject_Command.java,v $
 * Revision 1.6  2011/06/06 03:18:25  dick
 * JibsMessages belongs in n.s.jibs.server, not n.s.jibs.gui
 *
 * Revision 1.5  2011/06/03 19:27:28  dick
 * game.winGame[XO]() just needs the points, not the other parameters.
 *
 * Revision 1.4  2011/06/03 00:25:09  dick
 * Player.show2Watcher() becomes show2Watchers().
 *
 * Revision 1.3  2011/06/02 19:27:27  dick
 * Tell watchers about rejects.
 *
 * Revision 1.2  2011/05/31 13:02:42  dick
 * Bucko headers.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.command;

import net.sourceforge.jibs.backgammon.BackgammonBoard;
import net.sourceforge.jibs.backgammon.JibsGame;
import net.sourceforge.jibs.server.JibsMessages;
import net.sourceforge.jibs.server.JibsQuestion;
import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.server.Player;
import net.sourceforge.jibs.util.DoubleQuestion;
import net.sourceforge.jibs.util.JibsWriter;
import net.sourceforge.jibs.util.ResignQuestion;

/** Player rejects the other guy's resignation
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.6 $ <br> $Date: 2011/06/06 03:18:25 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/command/Reject_Command.java">cvs Reject_Command.java</a>
 */
public class Reject_Command implements JibsCommand {
	public boolean execute(JibsServer jibsServer, Player player,
			String strArgs, String[] args) {
		JibsMessages jibsMessages = jibsServer.getJibsMessages();
		JibsGame game = player.getGame();
		BackgammonBoard board = null;
		String msg = null;
		Object[] obj = null;
		JibsQuestion question = player.getQuestion();
		JibsWriter out = player.getOutputStream();
		Player opponent = null;

		if (game != null) {
			board = game.getBackgammonBoard();
			opponent = board.getOpponent(player);

			if (question instanceof DoubleQuestion) {
				DoubleQuestion doubleQuestion = (DoubleQuestion) question;
				int points = doubleQuestion.getCubeOld();

				if (points == 1) {
					obj = new Object[] { opponent.getName() };
					// m_you_giveup_1=You give up. %0 wins 1 point.
					msg = jibsMessages.convert("m_you_giveup_1", obj);
					out.println(msg);
					// m_other_giveup1=%0 gives up. You win 1 point.
					obj = new Object[] { player.getName() };
					msg = jibsMessages.convert("m_other_giveup1", obj);
					opponent.getOutputStream().println(msg);
					player.show2Watchers(msg);
					opponent.show2Watchers(msg);
				} else {
					obj = new Object[] {
							opponent.getName(), Integer.valueOf(points)
					};
					// m_you_giveup_1=You give up. %0 wins %1 points.
					msg = jibsMessages.convert("m_you_giveup_2", obj);
					out.println(msg);
					// m_other_giveup2=%0 gives up. You win %1 points.
					obj = new Object[] { player.getName(), Integer.valueOf(points) };
					msg = jibsMessages.convert("m_other_giveup2", obj);
					opponent.getOutputStream().println(msg);
					player.show2Watchers(msg);
					opponent.show2Watchers(msg);
				}

				if (board.isPlayerX(player)) {
					game.winGameO(points);
				} else {
					game.winGameX(points);
				}
			}

			if (question instanceof ResignQuestion) {
				// m_you_reject=You reject. The game continues.
				msg = jibsMessages.convert("m_you_reject");
				out.println(msg);
				// m_other_reject=%0 rejects. The game continues.
				obj = new Object[] { player.getName() };
				msg = jibsMessages.convert("m_other_reject", obj);
				opponent.getOutputStream().println(msg);
				player.show2Watchers(msg);
				opponent.show2Watchers(msg);
			}
		} else {
			// m_you_not_playing=** You're not playing.
			msg = jibsMessages.convert("m_you_not_playing");
			out.println(msg);
		}

		return true;
	}
}
