/******************************************************************************
 * Whois_Command.java - Handle the client's whois command.
 * $Id: Whois_Command.java,v 1.4 2011/06/06 03:18:25 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: Whois_Command.java,v $
 * Revision 1.4  2011/06/06 03:18:25  dick
 * JibsMessages belongs in n.s.jibs.server, not n.s.jibs.gui
 *
 * Revision 1.3  2011/06/06 02:58:39  dick
 * Use jibsServer.getPlayerFromDatabase() for that purpose.
 *
 * Revision 1.2  2011/05/31 13:02:42  dick
 * Bucko headers.
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.command;

import net.sourceforge.jibs.server.JibsMessages;
import net.sourceforge.jibs.server.JibsServer;
import net.sourceforge.jibs.server.Player;
import net.sourceforge.jibs.util.JibsWriter;

/** Handle a client's whois command.
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.4 $ <br> $Date: 2011/06/06 03:18:25 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/command/Whois_Command.java">cvs Whois_Command.java</a>
 */
public class Whois_Command implements JibsCommand {
	
	public boolean execute(JibsServer jibsServer, Player player,
			String strArgs, String[] args) {
		String msg = null;
		JibsWriter out = player.getOutputStream();
		JibsMessages jibsMessages = jibsServer.getJibsMessages();

		if (args.length == 2) {
			String name = args[1];
			Player whoisPlayer = jibsServer.getPlayerFromDatabase(name);

			if (whoisPlayer != null) {
				whoisPlayer.whois(out);
			} else {
				// m_noone=** There is no one called %0.
				Object[] obj = new Object[] { name };
				msg = jibsMessages.convert("m_noone", obj);
				out.println(msg);

				return true;
			}
		} else {
			// m_argument_mssing=** please give a name as an argument.
			msg = jibsMessages.convert("m_argument_mssing");
			out.println(msg);
		}

		return true;
	}
}
