/******************************************************************************
 * JibsMessages - Externalize the messages sent from jIBS to the client.
 * $Id: JibsMessages.java,v 1.2 2011/06/06 03:22:07 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: JibsMessages.java,v $
 * Revision 1.2  2011/06/06 03:22:07  dick
 * We don't have to spew to logger each of the messages loaded.  A count will suffice.
 *
 * Revision 1.1  2011/06/06 03:18:26  dick
 * JibsMessages belongs in n.s.jibs.server, not n.s.jibs.gui
 *
 * Revision 1.3  2011/05/31 14:16:52  dick
 * Fix cvs link.
 *
 * Revision 1.2  2011/05/29 05:17:40  dick
 * Read the messages text from the resources.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.server;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Map;
import java.util.StringTokenizer;


import org.apache.log4j.Logger;

/** Externalize the messages sent from jIBS to the client.
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.2 $ <br> $Date: 2011/06/06 03:22:07 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/gui/JibsMessages.java">cvs JibsMessages.java</a>
 */
public class JibsMessages {
	private final static boolean DEBUG = false; 
	private static Logger logger = Logger.getLogger(JibsMessages.class);
	private Map<String, String> messageMap = null;
	@SuppressWarnings("unused")
	private JibsServer jibsServer;

	public JibsMessages(JibsServer jibsServer, String filename) {
		BufferedReader buffer = null;

		try {
			this.jibsServer = jibsServer;
			//logger.info("Reading message file (" + filename + ")");

//			Reader reader = new FileReader(new File(filename));
//			buffer = new BufferedReader(reader);
			InputStream is = ClassLoader.getSystemResourceAsStream(filename);
			buffer = new BufferedReader(new InputStreamReader(is));

			messageMap = new HashMap<String, String>();

			String line;

			while ((line = buffer.readLine()) != null) {
				if (!line.startsWith("#")) {
					StringTokenizer stoken = new StringTokenizer(line, "=");

					while (stoken.hasMoreTokens()) {
						String par = stoken.nextToken();
						String value = "";

						if (stoken.hasMoreTokens()) {
							value = stoken.nextToken();

							// already defined ?
							if (messageMap.containsKey(par)) {
								logger.warn("" + par + " already defined.");
							}

							messageMap.put(par, value);
							if (DEBUG)
								logger.debug(par + "=" + value);
						}
					}
				}
			}
			logger.debug("Loaded " + messageMap.size() + "messages");
		} catch (FileNotFoundException e) {
			jibsServer.logException(e);
		} catch (IOException e) {
			jibsServer.logException(e);
		} finally {
			try {
				if (buffer != null) {
					buffer.close();
				}
			} catch (IOException e) {
				jibsServer.logException(e);
			}
		}
	}

	public String convert(String string) {
		return load(string);
	}

	public synchronized String convert(String string, Object[] parameter) {
		String s = load(string);

		StringBuffer buffer = new StringBuffer();
		StringBuffer number = null;
		boolean bNumber = false;

		for (int i = 0; i < s.length(); i++) {
			if (s.charAt(i) == '%') {
				number = new StringBuffer();
				bNumber = true;

				continue;
			} else {
				if (bNumber) {
					if (Character.isDigit(s.charAt(i))) {
						number.append(s.charAt(i));
					} else {
						bNumber = false;

						if (number != null) {
							int slot = Integer.parseInt(number.toString());

							if ((slot >= 0) && (slot < parameter.length)) {
								buffer.append(parameter[slot]);
							} else {
								buffer.append("%");
								buffer.append(number);
							}

							number = null;
						}

						buffer.append(s.charAt(i));
					}
				} else {
					buffer.append(s.charAt(i));
					number = null;
				}
			}
		}

		if (number != null) {
			int slot = Integer.parseInt(number.toString());

			if ((slot >= 0) && (slot < parameter.length)) {
				buffer.append(parameter[slot]);
			} else {
				buffer.append("%");
				buffer.append(number);
			}
		}

		return buffer.toString();
	}

	private String load(String ident) {
		String s = (String) messageMap.get(ident);

		if (s == null) {
			logger.warn("Message:\"" + ident + "\" not found.");
			s = "";
		}

		return s;
	}
}
